<?php
/**
 * PHPTAL templating engine
 *
 * PHP Version 5
 *
 * @category HTML
 * @package  PHPTAL
 * @author   Laurent Bedubourg <lbedubourg@motion-twin.com>
 * @author   Kornel Lesiński <kornel@aardvarkmedia.co.uk>
 * @license  http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @version  SVN: $Id: DefineMacro.php 794 2009-11-27 11:54:13Z kornel $
 * @link     http://phptal.org/
 */

namespace pear2\HTML\Template\PHPTAL\Php\Attribute\METAL; // PHPTAL_Php_Attribute_METAL_
use pear2\HTML\Template\PHPTAL as P;


/**
 * METAL Specification 1.0
 *
 *      argument ::= Name
 *
 * Example:
 *
 *      <p metal:define-macro="copyright">
 *      Copyright 2001, <em>Foobar</em> Inc.
 *      </p>
 *
 * PHPTAL:
 *
 *      <?php function XXX_macro_copyright($tpl) { ? >
 *        <p>
 *        Copyright 2001, <em>Foobar</em> Inc.
 *        </p>
 *      <?php } ? >
 *
 * @package PHPTAL
 * @subpackage Php.attribute.metal
 * @author Laurent Bedubourg <lbedubourg@motion-twin.com>
 */
class DefineMacro extends P\Php\Attribute
{
    public function before(P\Php\CodeWriter $codewriter)
    {
        $macroname = strtr(trim($this->expression), '-', '_');
        if (!preg_match('/^[a-z0-9_]+$/i', $macroname)) {
            throw new P\ParserException('Bad macro name "'.$macroname.'"', $this->phpelement->getSourceFile(), $this->phpelement->getSourceLine());
        }

        if ($codewriter->functionExists($macroname)) {
            throw new P\TemplateException("Macro $macroname is defined twice");
        }

        $codewriter->doFunction($macroname, 'P\PHPTAL $_thistpl, P\PHPTAL $tpl');
        $codewriter->doSetVar('$tpl', 'clone $tpl');
        $codewriter->doSetVar('$ctx', '$tpl->getContext()');
        $codewriter->doSetVar('$_translator', '$tpl->getTranslator()');
        $codewriter->doXmlDeclaration(true);
        $codewriter->doDoctype(true);
    }

    public function after(P\Php\CodeWriter $codewriter)
    {
        $codewriter->doEnd('function');
    }
}

