<?php
/**
 * PHPTAL templating engine
 *
 * PHP Version 5
 *
 * @category HTML
 * @package  PHPTAL
 * @author   Laurent Bedubourg <lbedubourg@motion-twin.com>
 * @author   Kornel Lesiński <kornel@aardvarkmedia.co.uk>
 * @license  http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @version  SVN: $Id: Condition.php 671 2009-07-11 18:11:35Z kornel $
 * @link     http://phptal.org/
 */

namespace pear2\HTML\Template\PHPTAL\Php\Attribute\TAL; // PHPTAL_Php_Attribute_TAL_
use pear2\HTML\Template\PHPTAL as P;


/**
 * TAL Specifications 1.4
 *
 *      argument ::= expression
 *
 * Example:
 *
 *      <p tal:condition="here/copyright"
 *         tal:content="here/copyright">(c) 2000</p>
 *
 *
 *
 *
 * @package PHPTAL
 * @subpackage Php.attribute.tal
 * @author Laurent Bedubourg <lbedubourg@motion-twin.com>
 */
class Condition
extends P\Php\Attribute
implements P\Php\TalesChainReader
{
    private $expressions = array();

    public function before(P\Php\CodeWriter $codewriter)
    {
        $code = $codewriter->evaluateExpression($this->expression);

        // If it's a chained expression build a new code path
        if (is_array($code)) {
            $this->expressions = array();
            $executor = new P\Php\TalesChainExecutor($codewriter, $code, $this);
            return;
        }

        // Force a falsy condition if the nothing keyword is active
        if ($code == P\Php\TalesInternal::NOTHING_KEYWORD) {
            $code = 'false';
        }

        $codewriter->doIf($code);
    }

    public function after(P\Php\CodeWriter $codewriter)
    {
        $codewriter->doEnd('if');
    }


    public function talesChainPart(P\Php\TalesChainExecutor $executor, $exp, $islast)
    {
        // check if the expression is empty
        if ($exp !== 'false') {
            $this->expressions[] = '!P\isempty(' . $exp . ')';
        }

        if ($islast) {
            // for the last one in the chain build a ORed condition
            $executor->getCodeWriter()->doIf( implode(' || ', $this->expressions ) );
            // The executor will always end an if so we output a dummy if
            $executor->doIf('false');
        }
    }

    public function talesChainNothingKeyword(P\Php\TalesChainExecutor $executor)
    {
        // end the chain
        $this->talesChainPart($executor, 'false', true);
        $executor->breakChain();
    }

    public function talesChainDefaultKeyword(P\Php\TalesChainExecutor $executor)
    {
        throw new P\ParserException('\'default\' keyword not allowed on conditional expressions');
    }

}


