<?php
/**
 * PHPTAL templating engine
 *
 * PHP Version 5
 *
 * @category HTML
 * @package  PHPTAL
 * @author   Laurent Bedubourg <lbedubourg@motion-twin.com>
 * @author   Kornel Lesiński <kornel@aardvarkmedia.co.uk>
 * @license  http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @version  SVN: $Id: Content.php 782 2009-11-02 09:57:17Z kornel $
 * @link     http://phptal.org/
 */

namespace pear2\HTML\Template\PHPTAL\Php\Attribute\TAL; // PHPTAL_Php_Attribute_TAL_
use pear2\HTML\Template\PHPTAL as P;



/** TAL Specifications 1.4
 *
 *     argument ::= (['text'] | 'structure') expression
 *
 * Example:
 *
 *     <p tal:content="user/name">Fred Farkas</p>
 *
 *
 *
 *
 * @package PHPTAL
 * @subpackage Php.attribute.tal
 * @author Laurent Bedubourg <lbedubourg@motion-twin.com>
 */
class Content
extends P\Php\Attribute
implements P\Php\TalesChainReader
{
    public function before(P\Php\CodeWriter $codewriter)
    {
        $expression = $this->extractEchoType($this->expression);

        $code = $codewriter->evaluateExpression($expression);

        if (is_array($code)) {
            return $this->generateChainedContent($codewriter, $code);
        }

        if ($code == P\Php\TalesInternal::NOTHING_KEYWORD) {
            return;
        }

        if ($code == P\Php\TalesInternal::DEFAULT_KEYWORD) {
            return $this->generateDefault($codewriter);
        }

        $this->doEchoAttribute($codewriter, $code);
    }

    public function after(P\Php\CodeWriter $codewriter)
    {
    }

    private function generateDefault(P\Php\CodeWriter $codewriter)
    {
        $this->phpelement->generateContent($codewriter,true);
    }

    protected function generateChainedContent(P\Php\CodeWriter $codewriter, $code)
    {
        $executor = new P\Php\TalesChainExecutor($codewriter, $code, $this);
    }

    public function talesChainPart(P\Php\TalesChainExecutor $executor, $exp, $islast)
    {
        if (!$islast) {
            $var = $executor->getCodeWriter()->createTempVariable();
            $executor->doIf('!P\isempty('.$var.' = '.$exp.')');
            $this->doEchoAttribute($executor->getCodeWriter(), $var);
            $executor->getCodeWriter()->recycleTempVariable($var);
        } else {
            $executor->doElse();
            $this->doEchoAttribute($executor->getCodeWriter(), $exp);
        }
    }

    public function talesChainNothingKeyword(P\Php\TalesChainExecutor $executor)
    {
        $executor->breakChain();
    }

    public function talesChainDefaultKeyword(P\Php\TalesChainExecutor $executor)
    {
        $executor->doElse();
        $this->generateDefault($executor->getCodeWriter());
        $executor->breakChain();
    }
}
