<?php

/**
 * Generates PHP that writes document in HTML5 mode.
 */
class PHPTAL_Php_Gen_HTML5 extends PHPTAL_Php_Gen_Standard
{
    public function visitXmlDeclaration(PHPTAL_Dom_XmlDeclaration $node)
    {
        // do nothing
    }

    public function visitDocumentType(PHPTAL_Dom_DocumentType $node)
    {
        $this->codewriter->setDocType('<!DOCTYPE html>');
        $this->codewriter->doDoctype();
    }

    public function visitCDATA(PHPTAL_Dom_CDATASection $node)
    {
        $value = $node->getValueEscaped();
        $inCDATAelement = PHPTAL_Dom_Defs::getInstance()->isCDATAElementInHTML($node->parentNode->getNamespaceURI(), $node->parentNode->getLocalName());

        // in HTML5 must limit it to <script> and <style>
        if ($inCDATAelement) {
            $this->codewriter->pushHTML($this->codewriter->interpolateCDATA(str_replace('</', '<\/', $value)));
        } else {
            $this->codewriter->pushHTML($this->codewriter->interpolateHTML(htmlspecialchars($value)));
        }
    }

    protected function openTag(PHPTAL_Dom_Element $node)
    {
        $this->codewriter->pushHTML('<'.$node->getLocalName());

        $this->generateAttributes($node);

        $this->codewriter->pushHTML('>');
    }

    protected function closeTag(PHPTAL_Dom_Element $node)
    {
        $this->codewriter->pushHTML('</'.$node->getLocalName().'>');
    }

    protected function generateAttribute(PHPTAL_Dom_Attr $attr)
    {
        // xmlns:foo is not allowed in text/html
        if ($attr->isNamespaceDeclaration()) {
            return;
        }

        switch ($attr->getReplacedState()) {
            case PHPTAL_Dom_Attr::NOT_REPLACED:
                $this->codewriter->pushHTML(' '.$attr->getQualifiedName());
                if (!PHPTAL_Dom_Defs::getInstance()->isBooleanAttribute($attr->getQualifiedName())) {
                    $html = $this->codewriter->interpolateHTML($attr->getValueEscaped());
                    $this->codewriter->pushHTML('='.$this->quoteAttributeValue($html));
                }
                break;

            case PHPTAL_Dom_Attr::HIDDEN:
                break;

            case PHPTAL_Dom_Attr::FULLY_REPLACED:
                $this->codewriter->pushHTML($attr->getValueEscaped());
                break;

            case PHPTAL_Dom_Attr::VALUE_REPLACED:
                $this->codewriter->pushHTML(' '.$attr->getQualifiedName().'="');
                $this->codewriter->pushHTML($attr->getValueEscaped());
                $this->codewriter->pushHTML('"');
                break;
        }
    }

    public function quoteAttributeValue($value)
    {
         // HTML 5: 8.1.2.3 Attributes ; http://code.google.com/p/html5lib/issues/detail?id=93
        if ($this->codewriter->getEncoding() == 'UTF-8') {
            // regex excludes unicode control characters, all kinds of whitespace and unsafe characters
            $attr_regex = '/^[^$&\/=\'"><\s`\pM\pC\pZ\p{Pc}\p{Sk}]+$/u'; // FIXME: interpolation is done _after_ that function, so $ must be forbidden for now
        } else {
            $attr_regex = '/^[^$&\/=\'"><\s`\0177-\377]+$/';
        }

        if (preg_match($attr_regex, $value)) {
            return $value;
        }
        else {
            return '"'.$value.'"';
        }
    }

    public function isEmptyNode(PHPTAL_Dom_Element $node)
    {
        return PHPTAL_Dom_Defs::getInstance()->isEmptyTagNS($node->getNamespaceURI(), $node->getLocalName());
    }
}
