<?php

/**
 * Generates PHP that writes document in XHTML mode.
 */
class PHPTAL_Php_Gen_XHTML extends PHPTAL_Php_Gen_Standard
{
    public function visitCDATA(PHPTAL_Dom_CDATASection $node)
    {
        $value = $node->getValueEscaped();
        $inCDATAelement = PHPTAL_Dom_Defs::getInstance()->isCDATAElementInHTML($node->parentNode->getNamespaceURI(), $node->parentNode->getLocalName());

        if ($inCDATAelement) {
            // in text/html "</" is dangerous and the only sensible way to escape is ECMAScript string escapes.
            $value = str_replace('</', '<\/', $value);

            $this->codewriter->pushHTML($this->codewriter->interpolateCDATA('<![CDATA['.$value.']]>'));
        } else {
            $this->codewriter->pushHTML($this->codewriter->interpolateHTML(htmlspecialchars($value)));
        }
    }

    public function visitText(PHPTAL_Dom_Text $node)
    {
        // TODO: write Text as CDATA here when needed? See PHPTAL_Dom_Element::replaceTextWithCDATA()

        parent::visitText($node);
    }

    protected function initElement(PHPTAL_Dom_Element $node)
    {
        parent::initElement($node);

        $node->replaceTextWithCDATA();
    }

    public function isEmptyNode(PHPTAL_Dom_Element $node)
    {
        return PHPTAL_Dom_Defs::getInstance()->isEmptyTagNS($node->getNamespaceURI(), $node->getLocalName());
    }
}
