<?php
/**
 * PHPTAL templating engine
 *
 * PHP Version 5
 *
 * @category HTML
 * @package  PHPTAL
 * @author   Kornel Lesiński <kornel@aardvarkmedia.co.uk>
 * @license  http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @version  SVN: $Id:$
 * @link     http://phptal.org/
 */

namespace pear2\HTML\Template\PHPTAL; // PHPTAL_
use pear2\HTML\Template\PHPTAL as P;

 
require_once dirname(__FILE__)."/config.php";

class IncludePathTest extends P\TestCase
{
    private $cwd;

    function testOverride()
    {
        $path = get_include_path();

        P\PHPTAL::setIncludePath();
        $this->assertNotEquals($path,get_include_path());
        P\PHPTAL::restoreIncludePath();
        $this->assertEquals($path,get_include_path());
    }

    function testPreserveCustomPath()
    {
        set_include_path(get_include_path() . \PATH_SEPARATOR . "./my-custom-path-test/");

        $this->assertContains("./my-custom-path-test/", $path = get_include_path());

        P\PHPTAL::setIncludePath();
        $this->assertNotEquals($path,get_include_path());
        P\PHPTAL::restoreIncludePath();
        $this->assertEquals($path,get_include_path());

        $this->assertContains("./my-custom-path-test/", get_include_path());
    }

    function testNesting()
    {
        $path = get_include_path();

        P\PHPTAL::setIncludePath();
        $this->assertNotEquals($path,get_include_path());

        P\PHPTAL::setIncludePath();
        $this->assertNotEquals($path,get_include_path());

        P\PHPTAL::setIncludePath();
        $this->assertNotEquals($path,get_include_path());


        P\PHPTAL::restoreIncludePath();
        $this->assertNotEquals($path,get_include_path());

        P\PHPTAL::restoreIncludePath();
        $this->assertNotEquals($path,get_include_path());

        P\PHPTAL::restoreIncludePath();
        $this->assertEquals($path,get_include_path());
    }

    function testIncludePathIsChanged()
    {
        $cwd = getcwd();
        chdir(dirname(__FILE__));
        try
        {
            $this->assertFileNotExists("./PHPTAL/Context.php");

            P\PHPTAL::setIncludePath();
            $modified_include_path = get_include_path();
            try
            {                
                $fp = @fopen("PHPTAL/Context.php","r",true);
                P\PHPTAL::restoreIncludePath();
            }
            catch(Exception $e)
            {
                P\PHPTAL::restoreIncludePath();
                
                if (!$e instanceof \ErrorException) throw $e;                
                $fp = null;
            }            

            $this->assertTrue(!!$fp, "File should be opened via include path: ".$modified_include_path.'; . = '.getcwd());
            
            if ($fp) fclose($fp);

            chdir($cwd);
        }
        catch(Exception $e)
        {
            chdir($cwd); throw $e;
        }
    }
}
